#define PYGGI_MODULE
#include "pyggi.h"

static PyObject*
Mode_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
	Mode *self;
	
	self = (Mode*)type->tp_alloc(type, 0);
	if (self!=NULL) {
		self->mode.frames     = GGI_AUTO;
		self->mode.graphtype  = GGI_AUTO;
		self->mode.visible.x  = GGI_AUTO;
		self->mode.visible.y  = GGI_AUTO;
		self->mode.virt.x     = GGI_AUTO;
		self->mode.virt.y     = GGI_AUTO;
		self->mode.dpp.x      = GGI_AUTO;
		self->mode.dpp.y      = GGI_AUTO;
		self->mode.size.x     = GGI_AUTO;
		self->mode.size.y     = GGI_AUTO;
	}
	return (PyObject*)self;
}

static int
Mode_init(Mode * self, PyObject *args, PyObject *kwds) {
	char *c;
	
	switch(PyTuple_Size(args)) {
	case 0:
		break;
	case 1:
		if(!PyString_Check(PyTuple_GET_ITEM(args,0))) {
			PyErr_SetString(PyExc_TypeError, "string required");
			return -1;
		}
		c = PyString_AsString(PyTuple_GET_ITEM(args,0));
		if (PyErr_Occurred())
			return -1;
		ggiParseMode(c,&(self->mode));
		break;
	default:
		PyErr_SetString(PyExc_TypeError,
				"Mode requires at most one arg");
		return -1;
	}
       	return 0;
}

static PyObject*
Mode_getattr(Mode *self, char *name) {
	switch(name[0]) {
	case 'f': 
		if(!strcmp(name,"frames"))
			return PyInt_FromLong((long)self->mode.frames);
		break;
	case 'v':
		if(!strcmp(name,"visible"))
			return Py_BuildValue("ii", self->mode.visible.x,
					     self->mode.visible.y);
		else if(!strcmp(name,"virtual"))
			return Py_BuildValue("ii", self->mode.virt.x,
					     self->mode.virt.y);
		break;
	case 's':
		if(!strcmp(name,"size"))
			return Py_BuildValue("ii", self->mode.size.x,
					     self->mode.size.y);
		break;
	case 'g':
		/* TODO : Improve graphtype */
		if(!strcmp(name,"graphtype")==0)
			return PyInt_FromLong((long)self->mode.graphtype);
		break;
	case 'd':
		if(strcmp(name,"dpp")==0)
			return Py_BuildValue("ii", self->mode.dpp.x,
					     self->mode.dpp.y);
		break;
	}
	PyErr_SetString(PyExc_AttributeError, name);
	return NULL;
}

static int
Mode_setattr(Mode *self, char *name, PyObject *value) {
	switch(name[0]) {
	case 'f': 
		if(!strcmp(name,"frames"))
			return pyggi_getInt(value, &self->mode.frames);
		break;
	case 'v':
		if(!strcmp(name,"visible"))
			return pyggi_getCoords(value, &self->mode.visible);
		if(!strcmp(name,"virtual"))
			return pyggi_getCoords(value, &self->mode.virt);
		break;
	case 's':
		if(!strcmp(name,"size"))
			return pyggi_getCoords(value, &self->mode.size);		
		break;
	case 'g':
		if(!strcmp(name,"graphtype"))
			return pyggi_getInt(value, &self->mode.graphtype);
		break;
	case 'd':
		if(!strcmp(name,"dpp"))
			return pyggi_getCoords(value, &self->mode.dpp);		
		break;
	} 
	PyErr_SetString(PyExc_AttributeError, name);
	return -1;
}

static PyObject*
Mode_str(Mode *self) {
	char s[80];
	ggiSPrintMode(s, &(self->mode));
	return PyString_FromString(s);
}

static PyObject*
Mode_repr(Mode * self) {
	char s[80];
	ggiSPrintMode(s, &(self->mode));
	return PyString_FromFormat("Mode('%s')",s);
}

static
int Mode_cmp(Mode *a, Mode *b) {

	/* XXX check that b is a Mode */

	if((a->mode.frames    == b->mode.frames)    &&
	   (a->mode.virt.x    == b->mode.virt.x)    &&
	   (a->mode.virt.y    == b->mode.virt.y)    &&
	   (a->mode.visible.x == b->mode.visible.x) &&
	   (a->mode.visible.y == b->mode.visible.y) &&
	   (a->mode.size.x    == b->mode.size.x)    &&
	   (a->mode.size.y    == b->mode.size.y)    &&
	   (a->mode.dpp.x     == b->mode.dpp.x)     &&
	   (a->mode.dpp.y     == b->mode.dpp.y)     &&
	   (a->mode.graphtype == b->mode.graphtype))
		return 0;
	return -1;
}

PyTypeObject ModeType = {
	PyObject_HEAD_INIT(NULL)
	0,                          /* ob_size         */
	"GGI._ggimodule.Mode",      /* tp_name         */
	sizeof(Mode),               /* tp_basicsize    */
	0,                          /* tp_itemsize     */
	0,                          /* tp_dealloc      */
	0,                          /* tp_print        */
	(getattrfunc)Mode_getattr,  /* tp_getattr      */
	(setattrfunc)Mode_setattr,  /* tp_setattr      */
	(cmpfunc)Mode_cmp,          /* tp_compare      */
	(reprfunc)Mode_repr,        /* tp_repr         */
	0,                          /* tp_as_number    */
	0,                          /* tp_as_sequence  */
	0,                          /* tp_as_mapping   */
	0,                          /* tp_hash         */
	0,                          /* tp_call         */
	(reprfunc)Mode_str,         /* tp_str          */
	0,                          /* tp_getattro     */
	0,                          /* tp_setattro     */
	0,                          /* tp_as_buffer    */
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,     /*tp_flags*/
	"Mode objects",             /* tp_doc          */
	
	/* 2.0 */
	0,		            /* tp_traverse     */
	0,		            /* tp_clear        */
	/* 2.1 */
	0,		            /* tp_richcompare  */
	0,		            /* tp_weaklistoffset */
	/* 2.2 */
	0,		            /* tp_iter         */
	0,		            /* tp_iternext     */
	0,                          /* tp_methods      */
	0,                          /* tp_members      */
	0,                          /* tp_getset       */
	0,                          /* tp_base         */
	0,                          /* tp_dict         */
	0,                          /* tp_descr_get    */
	0,                          /* tp_descr_set    */
	0,                          /* tp_dictoffset   */
	(initproc)Mode_init,        /* tp_init         */
	0,                          /* tp_alloc        */
	Mode_new,                   /* tp_new          */
	0,                          /* tp_is_gc        */
	0,                          /* tp_bases        */
        0,                          /* tp_mro          */
	0,                          /* tp_cache        */
	0,                          /* tp_subclasses   */
	0,                          /* tp_weaklist     */
	0                           /* tp_del          */
};
